@@ -7,19 +7,25 @@
 #ifndef _FCITX_CONFIG_CONFIGURATION_H_
 #define _FCITX_CONFIG_CONFIGURATION_H_
 
+#include <memory>
+#include <string>
+#include <vector>
 #include <fcitx-config/option.h>
+#include <fcitx-config/optiontypename.h>
+#include <fcitx-config/rawconfig.h>
 #include <fcitx-utils/macros.h>
 #include "fcitxconfig_export.h"
 
-#include <memory>
-
-#define FCITX_CONFIGURATION(NAME, ...)                                         \
+#define FCITX_CONFIGURATION_EXTEND(NAME, SUBCLASS, ...)                        \
     class NAME;                                                                \
     FCITX_SPECIALIZE_TYPENAME(NAME, #NAME)                                     \
-    FCITX_CONFIGURATION_CLASS(NAME, __VA_ARGS__)
+    FCITX_CONFIGURATION_CLASS_EXTEND(NAME, SUBCLASS, __VA_ARGS__)
 
-#define FCITX_CONFIGURATION_CLASS(NAME, ...)                                   \
-    class NAME : public ::fcitx::Configuration {                               \
+#define FCITX_CONFIGURATION(NAME, ...)                                         \
+    FCITX_CONFIGURATION_EXTEND(NAME, ::fcitx::Configuration, __VA_ARGS__)
+
+#define FCITX_CONFIGURATION_CLASS_EXTEND(NAME, SUBCLASS, ...)                  \
+    class NAME : public SUBCLASS {                                             \
     public:                                                                    \
         NAME() {}                                                              \
         NAME(const NAME &other) : NAME() { copyHelper(other); }                \
@@ -36,11 +42,9 @@
         __VA_ARGS__                                                            \
     };
 
-#define FCITX_OPTION(name, type, path, description, default, ...)              \
-    ::fcitx::Option<type> name {                                               \
-        this, std::string(path), std::string(description), (default),          \
-            __VA_ARGS__                                                        \
-    }
+#define FCITX_CONFIGURATION_CLASS(NAME, ...)                                   \
+    FCITX_CONFIGURATION_CLASS_EXTEND(NAME, ::fcitx::Configuration, __VA_ARGS__)
+
 namespace fcitx {
 
 class ConfigurationPrivate;
@@ -57,13 +61,23 @@ public:
     void load(const RawConfig &config, bool partial = false);
     void save(RawConfig &config) const;
     void dumpDescription(RawConfig &config) const;
-    virtual const char *typeName() const = 0;
+    FCITX_NODISCARD virtual const char *typeName() const = 0;
+
+    /**
+     * Set default value to current value.
+     *
+     * Sometimes, we need to customize the default value for the same type. This
+     * function will set the default value to current value.
+     */
+    void syncDefaultValueToCurrent();
 
 protected:
     bool compareHelper(const Configuration &other) const;
     void copyHelper(const Configuration &other);
 
 private:
+    void dumpDescriptionImpl(RawConfig &config,
+                             const std::vector<std::string> &parentPaths) const;
     void addOption(fcitx::OptionBase *option);
 
     FCITX_DECLARE_PRIVATE(Configuration);
